<?php

namespace LakeDrops\DrupalEnvironment;

use LakeDrops\Component\Composer\BaseHandler;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Handler class to setup LakeDrops tool based Drupal projects.
 *
 * @package LakeDrops\DrupalEnvironment
 */
class Handler extends BaseHandler {

  /**
   * {@inheritdoc}
   */
  public function configId(): string {
    return 'lakedrops';
  }

  /**
   * {@inheritdoc}
   */
  protected function configDefault(): array {
    return [
      'chg-acl' => FALSE,
      'webserver-username' => 'www-data',
      'webserver-groupname' => 'www-data',
      'filter_protocols' => [
        'http',
        'https',
        'ftp',
        'news',
        'nntp',
        'tel',
        'telnet',
        'mailto',
        'irc',
        'ssh',
        'sftp',
        'webcal',
        'rtsp',
      ],
      'cors_config' => [
        'enabled' => 'false',
        'allowedOrigins' => '*',
      ],
      'db' => [
        'driver' => 'mysql',
        'namespace' => 'Drupal\Core\Database\Driver\mysql',
        'database' => 'example',
        'username' => 'username',
        'password' => 'password',
        'host' => '127.0.0.1',
        'port' => '3306',
        'prefix' => '',
      ],
      'domain' => str_replace(['.', '-'], ['\\.', '\\-'], 'www.example.com'),
      'aliases' => [],
      'live' => (getenv('LAKEDROPS_BUILD_NG') === 'yes') || !empty($this->env->receiveGlobal('DRUPAL_LIVE', 'Live mode', '0')),
      'renderer_config' => [
        'debug' => $this->env->receiveGlobal('RENDERER_CONFIG_DEBUG', '', '0'),
      ],
      'file_assets_path' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function postInit(): void {
    $hash = $this->config->readValue('hash_salt');
    if ($hash === NULL) {
      try {
        $hash = str_replace(['+', '/', '='], [
          '-',
          '_',
          '',
        ], base64_encode(random_bytes(55)));
      }
      catch (\Exception) {
        $hash = 'no-random-bytes-available';
      }
    }
    $this->config->setValue('hash', $hash, FALSE);
    $this->config->setValue('php', ['version' => PHP_VERSION], FALSE);
  }

  /**
   * Post project create event to setup Drupal environment.
   */
  public function setupLakeDropsProject(): void {
    $this->init();

    $fs = new Filesystem();
    $installationManager = $this->composer->getInstallationManager();

    $drupalCorePackage = $this->getDrupalCorePackage();
    if ($drupalCorePackage === NULL) {
      return;
    }
    $corePath = $installationManager->getInstallPath($drupalCorePackage);
    $this->config->setValue('drupal8', version_compare($drupalCorePackage->getVersion(), '9.0.0', '<='), FALSE);

    // Directory where the root project is being created.
    $projectRoot = getcwd();
    // Directory where Drupal's index.php is located.
    $webRoot = basename(dirname($corePath));
    // Directory where this plugin is being installed.
    $pluginRoot = $installationManager->getInstallPath($this->getPackage('lakedrops/drupal-environment'));

    // Create and protect config, settings and files directories.
    foreach ([
      'settings/default' => TRUE,
      'files/default/files' => FALSE,
      'files/default/files/css' => FALSE,
      'files/default/files/js' => FALSE,
      'files/default/files/languages' => FALSE,
      'files/default/files/translations' => FALSE,
      'files/default/private' => FALSE,
      'drush' => TRUE,
    ] as $dir => $htaccess) {
      $path = $projectRoot . '/' . $dir;
      if (!$fs->exists($path)) {
        $fs->mkdir($path);
      }
      if ($htaccess) {
        $fs->copy($pluginRoot . '/templates/.htaccess', $path . '/.htaccess');
      }
      if ($this->config->readValue('chg-acl')) {
        $fs->chgrp($path, $this->config->readValue('webserver-groupname'), TRUE);
      }
    }

    // Link Drupal site's files and private directory.
    foreach (['files', 'private'] as $dir) {
      if (!$fs->exists($webRoot . '/sites/default/' . $dir)) {
        $rel = substr($fs->makePathRelative($projectRoot . '/files/default/' . $dir, $projectRoot . '/' . $webRoot . '/sites/default/' . $dir), 3, -1);
        $fs->symlink($rel, $webRoot . '/sites/default/' . $dir);
        $fs->chmod($webRoot . '/sites/default/' . $dir, 0775);
      }
    }

    // Link Drupal site's settings files.
    $settings_files = [
      'settings.php',
      'settings.local.php',
      'services.yml',
      'development.services.yml',
    ];
    foreach ($settings_files as $template) {
      $file = $projectRoot . '/settings/default/' . $template;
      $link = $webRoot . '/sites/default/' . $template;
      $rendered = $this->config->render($template, file_get_contents($pluginRoot . '/templates/' . $template . '.twig'));
      if (file_exists($file)) {
        $fs->chmod($file, 0664);
      }
      file_put_contents($file, $rendered);
      $rel = substr($fs->makePathRelative($file, $projectRoot . '/' . $link), 3, -1);
      $fs->symlink($rel, $link);
      $fs->chmod($file, 0444);
      if ($this->config->readValue('chg-acl')) {
        $fs->chgrp($file, $this->config->readValue('webserver-groupname'));
      }
    }

    // Link Drupal site's config directory.
    if (!$fs->exists($webRoot . '/sites/default/files/config')) {
      $rel = substr($fs->makePathRelative($projectRoot . '/config/default', $projectRoot . '/files/default/files/config'), 3, -1);
      $fs->symlink($rel, $webRoot . '/sites/default/files/config');
    }

    // Ensure default environment directories for config_split.
    foreach ([
      'develop',
      'test',
      'live',
      'nirvana',
      'snapshot',
    ] as $subdir) {
      $path = $projectRoot . '/config/default/environments/' . $subdir;
      if (!$fs->exists($path)) {
        $fs->mkdir($path);
      }
      $fs->copy($pluginRoot . '/templates/.htaccess', $path . '/.htaccess');
    }
  }

}
